# object classes - classes for game interactive objects
import pygame
import os.path

import utility

# base class for all objects
class GameObject:
	# initialization routine
	def __init__ (self, row, col, text, image = None, can_pickup = True):
		self.row = row
		self.col = col
		self.image = image
		self.text = text
		self.can_pickup = can_pickup

	# override this for interaction, i.e. when character walks into the item
	def interact (self):
		# always return True if the object is interacted with
		# return False to make the object block the player from
		# moving over it
		return True

	# use the object on another object
	def use (self, otherobject):
		pass

class GoldCoins (GameObject):
	# initialize
	def __init__ (self, row, col, image, value):
		text = "gold coins"
		self.value = value
		GameObject.__init__ (self, row, col, text, image, False)

	# no interaction with this object
	def interact (self):
		return True

	# use the object on Butaba - add to his gold
	def use (self, butaba):
		butaba.gold += self.value


class HealthPotion (GameObject):
	# initialize
	def __init__ (self, row, col, image):
		text = "health potion"
		GameObject.__init__ (self, row, col, text, image, True)

	# no interaction with this object
	def interact (self):
		return True

	# using the potion
	def use (self, butaba):
		butaba.health += 25
		if butaba.health > butaba.MAXHEALTH:
			butaba.health = butaba.MAXHEALTH

class Chest (GameObject):
	def __init__ (self, row, col, text, image, key_id, locked = False, objects = []):
		self.key_id = key_id
		self.locked = locked
		self.objects = objects
		GameObject.__init__ (self, row, col, text, image, False)

	# no interaction with this object. Also solid so return False
	def interact (self):
		# object is solid
		return False

	# try to use the key passed to it
	def use (self, key):
		# if chest is locked try to unlock it
		if self.locked is True:
			# if the item is a key
			if isinstance (key, Key):
				# if the key fits the lock
				# unlock the chest
				if key.key_id == self.key_id:
					self.locked = False
					# return the key
					return key
			# return None if not a key or key did not fit
			return None
		else:
			return self.locked

class Key (GameObject):
	def __init__ (self, row, col, text, image, key_id):
		self.key_id = key_id
		GameObject.__init__ (self, row, col, text, image, True)

	# no interaction with this object
	def interact (self):
		# key is not a solid object so return True
		return True

	# using the key - this is relegated to the locked item for
	# convenience, so key does nothing by defaults
	def use (self, lockitem):
		pass
