import pygame
import sys
import random
import os.path
import cPickle

import level
import butaba
import utility
import gameobjects
import constants

class MainGame:

	# initialize the game
	def __init__ (self):
		pygame.init ()
		self.screen  = pygame.display.set_mode ((720, 512))
		pygame.display.set_caption ("The Adventures of Butaba")

		# initalize background graphics
		self.img_tileset = pygame.image.load (os.path.join ("background", "tileset.png")).convert ()

		self.img_menu = pygame.image.load (os.path.join ("background", "menu_screen.png")).convert ()

		self.img_inventory = pygame.image.load (os.path.join ("background", "inventory.png")).convert ()

		self.img_chestbg = pygame.image.load (os.path.join ("background", "chestcontent.png")).convert ()


		# initialize object graphics
		self.img_redpotion = pygame.image.load (os.path.join ("objects", "red-potion.png")).convert ()
		self.img_redpotion.set_colorkey (pygame.Color (0, 255, 0))
		self.img_goldcoins = pygame.image.load (os.path.join ("objects", "gold-coins.png")).convert ()
		self.img_goldcoins.set_colorkey (pygame.Color (0, 255, 0))
		self.img_wand = pygame.image.load (os.path.join ("objects", "wand.png")).convert ()
		self.img_wand.set_colorkey (pygame.Color (0, 255, 0))
		self.img_bulb = pygame.image.load (os.path.join ("objects", "bulb.png")).convert ()
		self.img_bulb.set_colorkey (pygame.Color (0, 255, 0))
		self.img_lightning = pygame.image.load (os.path.join ("objects", "lightning.png")).convert ()
		self.img_lightning.set_colorkey (pygame.Color (0, 255, 0))
		self.img_key = pygame.image.load (os.path.join ("objects", "key.png")).convert ()
		self.img_key.set_colorkey (pygame.Color (0, 255, 0))
		self.img_key2 = pygame.image.load (os.path.join ("objects", "key2.png")).convert ()
		self.img_key2.set_colorkey (pygame.Color (0, 255, 0))
		self.img_chest = pygame.image.load (os.path.join ("objects", "chest.png")).convert ()
		self.img_chest.set_colorkey (pygame.Color (0, 255, 0))

		# initialize player graphics
		self.img_butabafront = pygame.image.load (os.path.join ("sprite", "butaba-front.png")).convert ()
		self.img_butabafront.set_colorkey (pygame.Color (0, 255, 0))
		self.img_butababack = pygame.image.load (os.path.join ("sprite", "butaba-back.png")).convert ()
		self.img_butababack.set_colorkey (pygame.Color (0, 255, 0))
		self.img_butabaleft = pygame.image.load (os.path.join ("sprite", "butaba-left.png")).convert ()
		self.img_butabaleft.set_colorkey (pygame.Color (0, 255, 0))
		self.img_butabaright = pygame.image.load (os.path.join ("sprite", "butaba-right.png")).convert ()
		self.img_butabaright.set_colorkey (pygame.Color (0, 255, 0))

		# set level data
		self.setup_levels ()
		# set current level and position of our character
		self.currentlevel = self.level1

		# set the status message
		self.status_message = "Game started"

		self.butaba = butaba.Butaba (5,0, butaba.Butaba.RIGHT)

	# set up the levels and their interactions
	def setup_levels (self):
		# set up the objects first
		chest1 = gameobjects.Chest (2, 6, "chest", self.img_chest, constants.KEY_CHEST1, True)
		chest2 = gameobjects.Chest (6, 6, "chest", self.img_chest, constants.KEY_CHEST2, True)
		key1 = gameobjects.Key (5, 3, "a chest key", self.img_key2, constants.KEY_CHEST1)
		key2 = 	gameobjects.Key (5, 3, "a chest key", self.img_key, constants.KEY_CHEST2)
		potion = gameobjects.HealthPotion (5, 2, self.img_redpotion)
		gold50 = gameobjects.GoldCoins (6, 2, self.img_goldcoins, 50)
		gold75 = gameobjects.GoldCoins (6, 2, self.img_goldcoins, 75)

		chest1.objects = [ gold50, gold75 ]

		# create the levels
		self.level1 = level.Level (cPickle.load (file ("levels/level1.dat")),
				objects = [ chest1 ] )

		self.level1w = level.Level (cPickle.load (file ("levels/level1w.dat")))

		self.level1e = level.Level (cPickle.load (file ("levels/level1e.dat")),
				objects = [ key1, key2, potion, chest2 ])

		# set up the interaction between levels
		self.level1.levelright = self.level1e
		self.level1.levelleft = self.level1w
		self.level1e.levelleft = self.level1
		self.level1w.levelright = self.level1

	def main_loop (self):
		# main game loop
		while 1:
			# clear screen
			self.screen.fill (pygame.Color (0,0,0))
			# draw the level
			self.draw_level_background (self.currentlevel)
			# draw level objects
			self.draw_level_objects (self.currentlevel)
			# draw our character
			self.draw_butaba ()
			# display the character's inventory
			self.draw_inventory ()
			# draw the status info
			self.draw_status ()
			# update the display
			pygame.display.update ()

			# get keyboard events
			for event in pygame.event.get ():
				if event.type == pygame.QUIT:
					sys.exit (0)
				# if keyboard event
				if event.type == pygame.KEYDOWN:
					if event.key == pygame.K_UP:
						self.move_butaba_up ()
					elif event.key == pygame.K_DOWN:
						self.move_butaba_down ()
					elif event.key == pygame.K_LEFT:
						self.move_butaba_left ()
					elif event.key == pygame.K_RIGHT:
						self.move_butaba_right ()
					# drinking health potion in inventory
					elif event.key == ord ("h") or event.key == ord ("H"):
						self.inventory_drink_health_potion ()
					# quit the game
					elif event.key == ord ("q") or event.key == ord ("Q"):
						sys.exit (0)

	# drink a health potion if it is in the player's inventory
	def inventory_drink_health_potion (self):
		# if health is maxed out then ignore
		if self.butaba.health == butaba.Butaba.MAXHEALTH:
			self.status_message = "You already have maximum health."
		else:
			# look for a health potion
			for item in self.butaba.inventory:
				if isinstance (item, gameobjects.HealthPotion) is True:
					self.use_object (self.butaba, item)
					break

	def move_butaba_up (self):
		# clear any status messages
		self.status_message = None
		# first if butaba is not facing up, make him face up
		if self.butaba.position <> butaba.Butaba.BACK:
			self.butaba.position = butaba.Butaba.BACK
			return

		# if butaba is trying to move off the top of the screen
		if self.butaba.row <= 0:
			# if there is a level above set current level to that one
			if self.currentlevel.leveltop is not None:
				lastrow = len (self.currentlevel.leveltop.background) - 1
				# if there is any object in that place interact with it
				# if any object is a blocking object then avoid movement
				if self.interact_objects (self.currentlevel.leveltop, lastrow, self.butaba.col) is False:
					return

				# make sure there is no obstacle
				if self.check_background_obstacle (self.currentlevel.leveltop, lastrow, self.butaba.col) is False:
					self.currentlevel = self.currentlevel.leveltop
					self.butaba.row = lastrow
		# normal upward movement
		else:
			# if there is any object in that place interact with it
			# if any object is a blocking object then avoid movement
			if self.interact_objects (self.currentlevel, self.butaba.row-1, self.butaba.col) is False:
				return

			if self.check_background_obstacle (self.currentlevel, self.butaba.row-1, self.butaba.col) is False:
				self.butaba.row -= 1

	def move_butaba_down (self):
		# clear any status messages
		self.status_message = None
		# first if butaba is not facing forward, make him face forward/down
		if self.butaba.position <> butaba.Butaba.FRONT:
			self.butaba.position = butaba.Butaba.FRONT
			return

		# if butaba is trying to move off the bottom of the screen
		if self.butaba.row >= len (self.currentlevel.background)-1:
			# if there is a level below set current level to that one
			if self.currentlevel.levelbottom is not None:
				# interact with objects if any
				# if any object is a blocking object then avoid movement
				if self.interact_objects (self.currentlevel.levelbottom, 0, self.butaba.col) is False:
					return
				# make sure there is no obstacle at that position
				if self.check_background_obstacle (self.currentlevel.levelbottom, 0, self.butaba.col) is False:
					self.currentlevel = self.currentlevel.levelbottom
					self.butaba.row = 0
		# normal downward movement
		else:
			# interact with objects if any
			# if any object is a blocking object then avoid movement
			if self.interact_objects (self.currentlevel, self.butaba.row+1, self.butaba.col) is False:
				return
			if self.check_background_obstacle (self.currentlevel, self.butaba.row+1, self.butaba.col) is False:
				self.butaba.row += 1

	# check if a background tile is an obstacle
	def check_background_obstacle (self, level, row, col):
		if (level.background[row][col][2] == 1):
			return True
		else:
			return False

	# get and interact with objects if present in a particular row/col
	def interact_objects (self, level, row, col):
		objs = []
		# get list of objects at current location
		for obj in level.objects:
			if obj.row == row and obj.col == col:
				objs.append (obj)

		# overall flag for blocking/non-blocking objects
		notblock = True
		# now perform interaction
		for obj in objs:
			# run the object interact function
			if obj.interact () is False:
				notblock = False
			# if object can be picked up ask
			if obj.can_pickup is True:
				ans = utility.ask_question (self.screen, "Found %s." % obj.text, ["Pick up", "Use", "Ignore"], self.img_menu)
				# if the answer is "pick up"
				if ans == 1:
					self.pickup_object (self.currentlevel, obj)
				elif ans == 2:
					# use the object according to its type
					self.use_object (self.currentlevel, obj)
			# if it cannot be picked up, try to use it anyway
			else:
				self.use_object (self.currentlevel, obj)

		return notblock

	# picking up an object
	def pickup_object (self, container, obj):
		# only if object can be picked up, pick it up or use it
		if obj.can_pickup is True:
			# check if the inventory is full
			if len (self.butaba.objects) >= butaba.Butaba.MAXITEMS:
				self.status_message = "Cannot pick up item. Inventory full"
			else:
				# add item to inventory
				self.butaba.objects.append (obj)
				# remove from container
				container.objects.remove (obj)

				self.status_message = "You picked up %s" % obj.text

	# this method uses the object first by calling the object use () method
	# and then performing specific actions as necessary
	def use_object (self, container, obj):
		# if the object is a health potion
		if isinstance (obj, gameobjects.HealthPotion) is True:
			obj.use (self.butaba)
			container.objects.remove (obj)
			self.status_message = "You gained health"
		# if the object is a chest
		elif isinstance (obj, gameobjects.Chest) is True:
			# if chest is locked, try to open it
			if obj.locked is True:
				# try opening the chest with every item 9the use () function
				# of the chest determines if item is a key anyway
				fittedkey = None
				for invobj in self.butaba.objects:
					fittedkey = obj.use (invobj)
					# if a key fits
					if fittedkey is not None:
						break
				# if no key found
				if fittedkey is None:
					self.status_message = "No key found to open %s" % obj.text
				# chest successfully unlocked
				else:
					self.status_message = "You unlocked the %s" % obj.text
					# remove the key from Butaba
					self.butaba.objects.remove (fittedkey)
					# add an experience point for unlocking chest subject
					# to a limit of KNOWLEDGEMAX_CHEST_UNLOCK
					if self.butaba.experience < constants.KNOWLEDGEMAX_CHEST_UNLOCK:
						self.butaba.experience += 1
						self.status_message += " and gained experience!"
			# display the contents of the chest
			else:
				utility.display_container_contents (self.screen, obj, self.img_chestbg, 30)

		# if the object is gold coins
		elif isinstance (obj, gameobjects.GoldCoins) is True:
			obj.use (self.butaba)
			self.status_message = "You picked up %d gold." % obj.value
			# remove the gold coins after adding it to Butaba's gold
			container.remove (obj)

	def move_butaba_left (self):
		# clear any status messages
		self.status_message = None

		# first if Butaba is not facing left, make him face left
		if self.butaba.position <> butaba.Butaba.LEFT:
			self.butaba.position = butaba.Butaba.LEFT
			return

		# if butaba is trying to move off the left edge
		if self.butaba.col <= 0:
			# if there is a level to the right set current level to that one
			if self.currentlevel.levelleft is not None:
				# get the last column of the previous level
				lastcol = len (self.currentlevel.levelleft.background[0]) - 1
				# interact with objects if any
				# if any object is a blocking object then avoid movement
				if self.interact_objects (self.currentlevel.levelleft, self.butaba.row, lastcol) is False:
					return
				# make sure there is no obstacle at that position of movement
				if self.check_background_obstacle (self.currentlevel.levelleft, self.butaba.row, lastcol) is False:
					self.currentlevel = self.currentlevel.levelleft
					self.butaba.col = lastcol
		# normal left movement
		else:
			# interact with objects if any
			# if any object is a blocking object then avoid movement
			if self.interact_objects (self.currentlevel, self.butaba.row, self.butaba.col-1) is False:
				return
			if self.check_background_obstacle (self.currentlevel, self.butaba.row, self.butaba.col-1) is False:
				self.butaba.col -= 1

	def move_butaba_right (self):
		# clear any status messages
		self.status_message = None

		# First if Butaba is not facing right make him face right
		if self.butaba.position <> butaba.Butaba.RIGHT:
			self.butaba.position = butaba.Butaba.RIGHT
			return

		# if butaba is trying to move off the right edge
		if self.butaba.col >= len (self.currentlevel.background[0])-1:
			# if there is a level to the right swap current level with that one
			if self.currentlevel.levelright is not None:
				# interact with objects if any
				# if any object is a blocking object then avoid movement
				if self.interact_objects (self.currentlevel.levelright, self.butaba.row, 0) is False:
					return

				# make sure there is no obstacle at that position of movement
				# get the last column of the previous level
				if self.check_background_obstacle (self.currentlevel.levelright, self.butaba.row, 0) is False:
					self.currentlevel = self.currentlevel.levelright
					self.butaba.col = 0
		# normal right movement
		else:
			# interact with objects if any
			# if any object is a blocking object then avoid moving
			if self.interact_objects (self.currentlevel, self.butaba.row, self.butaba.col + 1) is False:
				return
			if self.check_background_obstacle (self.currentlevel, self.butaba.row, self.butaba.col + 1) is False:
				self.butaba.col += 1

	def draw_butaba (self):
		if self.butaba.position == butaba.Butaba.FRONT:
			self.screen.blit (self.img_butabafront, (self.butaba.col*48, self.butaba.row*48))
		elif self.butaba.position == butaba.Butaba.BACK:
			self.screen.blit (self.img_butababack, (self.butaba.col*48, self.butaba.row*48))
		elif self.butaba.position == butaba.Butaba.LEFT:
			self.screen.blit (self.img_butabaleft, (self.butaba.col*48, self.butaba.row*48))
		elif self.butaba.position == butaba.Butaba.RIGHT:
			self.screen.blit (self.img_butabaright, (self.butaba.col*48, self.butaba.row*48))


	# Draw the status infodisplay
	def draw_status (self):
		self.screen.blit (self.img_redpotion, (485, 10))
		utility.put_text (self.screen, 550, 25, 20, (255, 0, 0), "%d" % self.butaba.health)

		self.screen.blit (self.img_lightning, (620, 10))
		utility.put_text (self.screen, 660, 25, 20, (255,255,255), "%d" % self.butaba.strength)

		self.screen.blit (self.img_wand, (485, 65))
		utility.put_text (self.screen, 550, 75, 20, (0, 0, 255), "%d" % self.butaba.magic)

		self.screen.blit (self.img_bulb, (620, 65))
		utility.put_text (self.screen, 660, 75, 20, (0, 255, 0), "%d" % self.butaba.experience)

		self.screen.blit (self.img_goldcoins, (485, 110))
		utility.put_text (self.screen, 550, 130, 20, (255, 255, 0), "%d" % self.butaba.gold)

		if self.status_message is not None:
			utility.put_text (self.screen, 10, 485, 10, (255,255, 0), "%s" % self.status_message)

	# display the inventory of the player
	def draw_inventory (self):
		# draw the inventory slots
		r = 1
		c = 1
		utility.put_text (self.screen, 490, 170, 16, (255,255 , 0), "Inventory")
		for i in range (butaba.Butaba.MAXITEMS):
			self.screen.blit (self.img_inventory, (440+c*54, 150+r*54))
			if c % 4 == 0:
				r += 1
				c = 1
			else:
				c += 1

		r = 1
		c = 1
		for obj in self.butaba.objects:
			self.screen.blit (obj.image, (440+c*54+2, 150+r*54+2))
			if c % 4 == 0:
				r += 1
				c = 1
			else:
				c += 1

	# Draw the level background tiles on surface
	def draw_level_background (self, level):
		i = 0
		for row in level.background:
			j = 0
			for tilerow, tilecol, is_solid in row:
				tilex = tilecol * 48
				tiley = tilerow * 48
				self.screen.blit (self.img_tileset, (j*48, i*48), pygame.Rect (tilex, tiley, 48, 48))

				j += 1
			i += 1

	# Draw the level objects
	def draw_level_objects (self, level):
		for obj in level.objects:
			if obj.image is not None:
				self.screen.blit (obj.image, (obj.col*48, obj.row*48))
